"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const apputils_1 = require("@jupyterlab/apputils");
const widgets_1 = require("@phosphor/widgets");
const datagrid_1 = require("@phosphor/datagrid");
const commands_1 = require("@phosphor/commands");
const services_1 = require("./services");
var Options;
(function (Options) {
    Options.unselectedBackgroundColor = 'white';
    Options.selectedBackgroundColor = '#2196f3';
    Options.unselectedTextColor = 'black';
    Options.selectedTextColor = 'white';
})(Options || (Options = {}));
var CommandIds;
(function (CommandIds) {
    CommandIds.copyToClipboard = 'copy-selection-to-clipboard';
})(CommandIds || (CommandIds = {}));
class ResponseTable {
    constructor(model) {
        this._isDisposed = false;
        this._grid = new datagrid_1.DataGrid();
        this._grid.model = model;
        this._selectionManager = new services_1.DataGridExtensions.SelectionManager(model);
        this._onClick = this._onClick.bind(this);
        this._onContextMenu = this._onContextMenu.bind(this);
        this._copySelectionToClipboard = this._copySelectionToClipboard.bind(this);
        this._updateRenderers();
        this._selectionManager.selectionChanged.connect(() => {
            this._updateRenderers();
        });
        this._menu = this._createContextMenu();
        this._clickEventHandler = services_1.DataGridExtensions.addMouseEventListener('click', this._grid, this._onClick);
        this._contextMenuEventHandler = services_1.DataGridExtensions.addMouseEventListener('contextmenu', this._grid, this._onContextMenu);
    }
    static fromKeysRows(keys, data) {
        const model = new ResponseTableDataModel(keys, data);
        return new ResponseTable(model);
    }
    get widget() {
        return this._grid;
    }
    dispose() {
        this._clickEventHandler.dispose();
        this._contextMenuEventHandler.dispose();
        this._grid.dispose();
        this._menu.dispose();
        this._isDisposed = true;
    }
    get isDisposed() {
        return this._isDisposed;
    }
    _onClick(event) {
        const { row, column } = event;
        this._updateSelection(row, column);
    }
    _onContextMenu(event) {
        const { row, column, rawEvent } = event;
        this._updateSelection(row, column);
        if (this._isInBody(row, column)) {
            this._menu.open(rawEvent.clientX, rawEvent.clientY);
            rawEvent.preventDefault();
        }
    }
    _updateSelection(row, column) {
        if (this._isInBody(row, column)) {
            this._selectionManager.selection = {
                rowIndex: row.index,
                columnIndex: column.index
            };
        }
        else {
            this._selectionManager.selection = null;
        }
    }
    _isInBody(row, column) {
        return (row.section === 'row' &&
            column.section === 'column' &&
            row.index !== null &&
            column.index !== null);
    }
    _updateRenderers() {
        const renderer = this._textRendererForSelection(this._selectionManager.selection);
        this._grid.cellRenderers.set('body', {}, renderer);
    }
    _textRendererForSelection(selectedCell) {
        let backgroundColor;
        let textColor;
        if (selectedCell === null) {
            backgroundColor = Options.unselectedBackgroundColor;
            textColor = Options.unselectedTextColor;
        }
        else {
            const selectedRow = selectedCell.rowIndex;
            const selectedColumn = selectedCell.columnIndex;
            backgroundColor = ({ row, column }) => {
                if (row === selectedRow && column === selectedColumn) {
                    return Options.selectedBackgroundColor;
                }
                else {
                    return Options.unselectedBackgroundColor;
                }
            };
            textColor = ({ row, column }) => {
                if (row === selectedRow && column === selectedColumn) {
                    return Options.selectedTextColor;
                }
                else {
                    return Options.unselectedTextColor;
                }
            };
        }
        return new datagrid_1.TextRenderer({ backgroundColor, textColor });
    }
    _createContextMenu() {
        const commands = new commands_1.CommandRegistry();
        commands.addCommand(CommandIds.copyToClipboard, {
            label: 'Copy cell',
            iconClass: 'jp-MaterialIcon jp-CopyIcon',
            execute: this._copySelectionToClipboard
        });
        const menu = new widgets_1.Menu({ commands });
        menu.addItem({ command: CommandIds.copyToClipboard });
        return menu;
    }
    _copySelectionToClipboard() {
        const selection = this._selectionManager.selection;
        if (selection !== null) {
            const { rowIndex, columnIndex } = selection;
            const value = this._grid.model.data('body', rowIndex, columnIndex);
            apputils_1.Clipboard.copyToSystem(value);
        }
    }
}
exports.ResponseTable = ResponseTable;
class ResponseTableDataModel extends datagrid_1.DataModel {
    constructor(keys, data) {
        super();
        this._data = data;
        this._keys = keys;
    }
    rowCount(region) {
        return region === 'body' ? this._data.length : 1;
    }
    columnCount(region) {
        return region === 'body' ? this._keys.length : 1;
    }
    data(region, row, column) {
        if (region === 'row-header') {
            return row;
        }
        if (region === 'column-header') {
            return this._keys[column];
        }
        if (region === 'corner-header') {
            return '';
        }
        return this._serializeData(this._data[row][column]);
    }
    _serializeData(data) {
        const _type = typeof data;
        if (_type === 'object') {
            return JSON.stringify(data);
        }
        return data;
    }
}
